@@ -116,7 +116,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"C_setcolorder", (DL_FUNC) &setcolorder, 2},
   {"C_subsetCols", (DL_FUNC) &subsetCols, 3},
   {"C_alloc", (DL_FUNC) &falloc, 3},
-  {"C_frange", (DL_FUNC) &frange, 2},
+  {"C_frange", (DL_FUNC) &frange, 3},
   {"C_fdist", (DL_FUNC) &fdist, 4},
   {"C_fnrow", (DL_FUNC) &fnrowC, 1},
   {"C_createeptr", (DL_FUNC) &createeptr, 1},
@@ -131,6 +131,8 @@ static const R_CallMethodDef CallEntries[] = {
   {"C_pivot_wide", (DL_FUNC) &pivot_wide, 5},
   {"C_sort_merge_join", (DL_FUNC) &sort_merge_join, 4},
   {"C_replace_outliers", (DL_FUNC) &replace_outliers, 5},
+  {"C_na_locf", (DL_FUNC) &na_locf, 2},
+  {"C_na_focb", (DL_FUNC) &na_focb, 2},
   // {"C_aschar", (DL_FUNC) &CasChar, 1},
   {"C_subsetDT", (DL_FUNC) &subsetDT, 4},
   {"C_subsetVector", (DL_FUNC) &subsetVector, 3},
